# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
spyder.plugins.plots
====================

Plots plugin
"""

from spyder.plugins.plots.plugin import Plots

# The following statement is required to be able to grab internal plugins.
PLUGIN_CLASSES = [Plots]
